@@ -21,7 +21,7 @@ use Digest::MD5;
 use MYDan;
 use MYDan::Agent::FileCache;
 
-my %index;
+my ( %index, %connect );
 
 sub new
 {
@@ -77,7 +77,7 @@ sub run
 
     my $term = AnyEvent->signal (signal => "TERM", cb => $excb );
     my $ints = AnyEvent->signal (signal => "INT",  cb => $excb );
-    my $usr1 = AnyEvent->signal (signal => "USR1", cb => sub{ print Dumper \%index; } );
+    my $usr1 = AnyEvent->signal (signal => "USR1", cb => sub{ print Dumper +{ index => \%index, connect => \%connect } } );
 
     my %savecb;
     my $childcb = sub
@@ -255,6 +255,84 @@ sub run
            on_read => sub {
                my $self = shift;
 
+               if( ! $index{$index}{rbuf} && $self->{rbuf} =~ s/^MYDanConnect_::([a-zA-Z0-9]{32})::_MYDanConnect// )
+               {
+                   my $uuid = $1;
+
+                   close $tmp_handle;
+                   unlink "$tmp/$index" if -e "$tmp/$index";
+
+                   unless( $connect{$uuid} )
+                   {
+                       if( keys( %connect ) >= $max )
+                       {
+                            my $c = delete $index{$index};
+                            $c->{handle}->push_write("MYDan connect limit");
+                            $c->{handle}->on_drain(undef);
+                            $c->{handle}->push_shutdown;
+                            $c->{handle}->destroy() unless $c->{handle}->destroyed();
+                            return;
+                       }
+ 
+                       $connect{$uuid}{$index} = delete $index{$index};
+                       $connect{$uuid}{$index}{handle}->on_read( undef );
+                       $connect{$uuid}{$index}{handle}->on_drain( undef );
+                       $connect{$uuid}{time} = time;
+                       return;
+                   }
+
+                   if( $connect{$uuid} && keys( %{$connect{$uuid}} ) >= 3 )
+                   {
+                       $index{$index}{handle}->push_write("uuidRepeat");
+                       $index{$index}{handle}->on_drain(undef);
+                       $index{$index}{handle}->push_shutdown;
+                       $index{$index}{handle}->destroy() unless $index{$index}{handle}->destroyed();
+                       return;
+                   }
+
+                   $connect{$uuid}{$index} = delete $index{$index};
+
+                   for my $id ( grep{ $_ ne 'time' }keys %{$connect{$uuid}} )
+                   {
+                       my ( $bid ) = grep{ $_ ne $id }grep{ $_ ne 'time' } keys %{$connect{$uuid}};
+
+                       $connect{$uuid}{$id}{on_drain} = sub{
+                           $connect{$uuid}{$bid}{handle}->on_read( $connect{$uuid}{$bid}{on_read} );
+                       };
+                       $connect{$uuid}{$id}{on_read} = sub{
+                           $connect{$uuid}{$id}{handle}->on_read( undef );
+                           my $self = shift;
+                           my $len = length $self->{rbuf};
+                           $self->push_read (
+                               chunk => $len,
+                               sub { $connect{$uuid}{$bid}{handle}->push_write($_[1]); }
+                           );
+                       };
+
+                       my $erc = sub{
+                           map{
+                           $connect{$uuid}{$_}{handle}->on_drain(undef);
+                           $connect{$uuid}{$_}{handle}->push_shutdown;
+                           $connect{$uuid}{$_}{handle}->destroy() unless $connect{$uuid}{$_}{handle}->destroyed();
+
+                           }( $id, $bid );
+                           delete $connect{$uuid} if $connect{$uuid}{$bid}{handle}->destroyed() 
+                           && $connect{$uuid}{$id}{handle}->destroyed();
+                       };
+
+                       $connect{$uuid}{$id}{handle}->on_error( $erc  );
+                       $connect{$uuid}{$id}{handle}->on_eof( $erc );
+                   }
+
+                   for my $id ( grep{ $_ ne 'time' }keys %{$connect{$uuid}} )
+                   {
+                       $connect{$uuid}{$id}{handle}->on_read( $connect{$uuid}{$id}{on_read} );
+                       $connect{$uuid}{$id}{handle}->on_drain( $connect{$uuid}{$id}{on_drain} );
+                   }
+
+                   return;
+               }
+
                if( ! $index{$index}{rbuf} && $self->{rbuf} =~ s/^MYDanExtractFile_::(\d+):(\d+):([a-zA-Z0-9]{32}):([a-zA-Z0-9\/\._\-]+)::_MYDanExtractFile// )
                {
                     my ( $qsize, $esize, $md5, $aim  ) = ( $1, $2, $3, $4 );
@@ -371,7 +449,7 @@ sub run
     
     };
 
-    my $t = AnyEvent->timer(
+    my $ti = AnyEvent->timer(
         after => 1, 
         interval => 1,
         cb => sub { 
@@ -381,6 +459,29 @@ sub run
         }
     ); 
 
+    my $tc = AnyEvent->timer(
+        after => 1, 
+        interval => 1,
+        cb => sub { 
+            for my $uuid ( keys %connect )
+            {
+                my $len = scalar keys %{$connect{$uuid}};
+
+                next if ( $len == 2 && $connect{$uuid}{time} + 6 > time ) || ( $len == 3 && $connect{$uuid}{time} + 3600 > time );
+                my $conn = delete $connect{$uuid};
+
+                for my $uuid ( grep{ $_ ne 'time' }keys %$conn )
+                {
+                    $conn->{$uuid}{handle}->on_drain(undef);
+                    $conn->{$uuid}{handle}->on_read(undef);
+                    $conn->{$uuid}{handle}->push_shutdown;
+                    $conn->{$uuid}{handle}->destroy() unless $conn->{$uuid}{handle}->destroyed();
+                }
+
+            }
+        }
+    ); 
+
     my $mtime = 0;
     my $wl = AnyEvent->timer(
         after => 1, 
